import os
from flask_covid19.blueprints.ecdc.ecdc_model_import import EcdcImport
from flask_covid19.blueprints.who.who_service_import import WhoImport
from flask_covid19.blueprints.rki_vaccination.rki_vaccination_model_import import RkiVaccinationImport
from flask_covid19.blueprints.owid.owid_model_import import OwidImport
from flask_covid19.blueprints.rki_cases.rki_model_import import RkiImport
from flask_covid19.blueprints.divi.divi_model_import import DiviImport


class ApplicationServiceConfig:
    def __init__(self, slug: str,
                 category: str,
                 cvsfile_subpath: str,
                 sub_category: str,
                 tablename: str,
                 cvsfile_name: str,
                 url_src: str):
        self.limit_nr = 20
        self.data_path = ".." + os.sep + "data"
        self.slug = slug,
        self.category = category
        self.cvsfile_subpath = cvsfile_subpath
        self.sub_category = sub_category
        self.tablename = tablename
        self.cvsfile_name = cvsfile_name
        self.url_src = url_src
        self.download_path = self.data_path + os.sep + self.cvsfile_name
        self.cvsfile_path = self.data_path + os.sep + self.cvsfile_subpath + os.sep + self.cvsfile_name
        self.msg_job = "download FILE: "+self.cvsfile_name+" from "+self.url_src
        self.msg_ok = "downloaded FILE: " + self.cvsfile_path + " from " + self.url_src
        self.msg_error = "Error while downloading: " + self.cvsfile_path + " from " + self.url_src

    @classmethod
    def create_config_for_who(cls):
        return ApplicationServiceConfig(
            slug='who',
            category='WHO',
            cvsfile_subpath='who',
            sub_category='Cases and Deaths',
            tablename=WhoImport.__tablename__,
            cvsfile_name="WHO-COVID-19-global-data.csv",
            url_src="https://covid19.who.int/WHO-COVID-19-global-data.csv",
        )

    @classmethod
    def create_config_for_rki_vaccination(cls):
        return ApplicationServiceConfig(
            slug='rki_vaccination',
            category='RKI',
            cvsfile_subpath='rki_vaccination',
            sub_category='Vaccination',
            tablename=RkiVaccinationImport.__tablename__,
            cvsfile_name="germany_vaccinations_timeseries_v2.tsv",
            url_src="https://impfdashboard.de/static/data/germany_vaccinations_timeseries_v2.tsv",
        )

    @classmethod
    def create_config_for_owid(cls):
        return ApplicationServiceConfig(
            slug='owid',
            category='OWID',
            cvsfile_subpath='owid',
            sub_category='Our World in Data',
            tablename=OwidImport.__tablename__,
            cvsfile_name="owid-covid-data.csv",
            url_src='"https://covid.ourworldindata.org/data/owid-covid-data.csv"',
        )

    @classmethod
    def create_config_for_ecdc(cls):
        return ApplicationServiceConfig(
            slug='ecdc',
            category='ECDC',
            cvsfile_subpath='ecdc',
            sub_category='European Centre for Disease Prevention and Control',
            tablename=EcdcImport.__tablename__,
            cvsfile_name="ecdc_europa_data.csv",
            url_src="https://opendata.ecdc.europa.eu/covid19/casedistribution/csv/",
        )

    @classmethod
    def create_config_for_rki(cls):
        # https://www.arcgis.com/home/item.html?id=f10774f1c63e40168479a1feb6c7ca74
        return ApplicationServiceConfig(
            slug='rki',
            category='RKI',
            cvsfile_subpath='rki',
            sub_category='Cases',
            tablename=RkiImport.__tablename__,
            cvsfile_name="RKI_COVID19.csv",
            url_src="https://www.arcgis.com/sharing/rest/content/items/f10774f1c63e40168479a1feb6c7ca74/data",
        )

    @classmethod
    def create_config_for_intensivregister(cls):
        return ApplicationServiceConfig(
            slug='divi',
            category='DIVI',
            cvsfile_subpath='divi',
            sub_category='Intensivregister',
            tablename=DiviImport.__tablename__,
            cvsfile_name="DIVI_intensivregister.json",
            url_src="https://www.intensivregister.de/api/public/intensivregister",
        )
