from flask import render_template, redirect, url_for, flash, Blueprint
from celery import states
from celery.utils.log import get_task_logger
from flask_admin.contrib.sqla import ModelView

from database import admin, db
from flask_covid19 import who_service, owid_service
from flask_covid19 import ecdc_service
from flask_covid19.blueprints.app_mq.application_workers import celery

from flask_covid19 import EcdcImport
from flask_covid19 import EcdcDateReported, EcdcContinent, EcdcCountry, EcdcData
from flask_covid19 import ApplicationPage


app_ecdc = Blueprint('ecdc', __name__, template_folder='templates', url_prefix='/ecdc')

admin.add_view(ModelView(EcdcImport, db.session, category="ECDC"))
admin.add_view(ModelView(EcdcDateReported, db.session, category="ECDC"))
admin.add_view(ModelView(EcdcContinent, db.session, category="ECDC"))
admin.add_view(ModelView(EcdcCountry, db.session, category="ECDC"))
admin.add_view(ModelView(EcdcData, db.session, category="ECDC"))


# ---------------------------------------------------------------------------------------------------------------
#  Url Routes Frontend
# ---------------------------------------------------------------------------------------------------------------

@app_ecdc.route('/info')
def url_ecdc_info():
    page_info = ApplicationPage('Europe', "Info")
    return render_template(
        'ecdc/ecdc_info.html',
        title='Europe',
        page_info=page_info)


@app_ecdc.route('/tasks')
def url_ecdc_tasks():
    page_info = ApplicationPage('Europe', "Tasks")
    return render_template(
        'ecdc/ecdc_tasks.html',
        title='Europe Tasks',
        page_info=page_info)


@app_ecdc.route('/imported/page/<int:page>')
@app_ecdc.route('/imported')
def url_ecdc_data_imported(page=1):
    page_info = ApplicationPage('Europe', "Last Import")
    page_data = EcdcImport.get_all_as_page(page)
    return render_template(
        'ecdc/imported/ecdc_imported.html',
        page_data=page_data,
        page_info=page_info)


@app_ecdc.route('/date_reported/all/page/<int:page>')
@app_ecdc.route('/date_reported/all')
def url_ecdc_date_reported_all(page=1):
    page_info = ApplicationPage('Europe', "date_reported")
    page_data = EcdcDateReported.get_all_as_page(page)
    return render_template(
        'ecdc/date_reported/ecdc_date_reported_all.html',
        page_data=page_data,
        page_info=page_info)


@app_ecdc.route('/date_reported/<int:europe_date_reported_id>/page/<int:page>')
@app_ecdc.route('/date_reported/<int:europe_date_reported_id>')
@app_ecdc.route('/date_reported/notification_rate/<int:europe_date_reported_id>/page/<int:page>')
@app_ecdc.route('/date_reported/notification_rate/<int:europe_date_reported_id>')
def url_ecdc_date_reported_one_notification_rate(europe_date_reported_id, page=1):
    page_info = ApplicationPage('Europe', "date_reported")
    europe_date_reported = EcdcDateReported.get_by_id(europe_date_reported_id)
    page_data = EcdcData.find_by_date_reported_notification_rate(europe_date_reported, page)
    return render_template(
        'ecdc/date_reported/ecdc_date_reported_one_notification_rate.html',
        europe_date_reported=europe_date_reported,
        page_data=page_data,
        page_info=page_info)


@app_ecdc.route('/date_reported/deaths_weekly/<int:europe_date_reported_id>/page/<int:page>')
@app_ecdc.route('/date_reported/deaths_weekly/<int:europe_date_reported_id>')
def url_ecdc_date_reported_one_deaths_weekly(europe_date_reported_id, page=1):
    page_info = ApplicationPage('Europe', "date_reported")
    europe_date_reported = EcdcDateReported.get_by_id(europe_date_reported_id)
    page_data = EcdcData.find_by_date_reported_deaths_weekly(europe_date_reported, page)
    return render_template(
        'ecdc/date_reported/ecdc_date_reported_one_deaths_weekly.html',
        europe_date_reported=europe_date_reported,
        page_data=page_data,
        page_info=page_info)


@app_ecdc.route('/date_reported/cases_weekly/<int:europe_date_reported_id>/page/<int:page>')
@app_ecdc.route('/date_reported/cases_weekly/<int:europe_date_reported_id>')
def url_ecdc_date_reported_one_cases_weekly(europe_date_reported_id, page=1):
    page_info = ApplicationPage('Europe', "date_reported")
    europe_date_reported = EcdcDateReported.get_by_id(europe_date_reported_id)
    page_data = EcdcData.find_by_date_reported_cases_weekly(europe_date_reported, page)
    return render_template(
        'ecdc/date_reported/ecdc_date_reported_one_cases_weekly.html',
        europe_date_reported=europe_date_reported,
        page_data=page_data,
        page_info=page_info)


@app_ecdc.route('/continent/all/page/<int:page>')
@app_ecdc.route('/continent/all')
def url_ecdc_continent_all(page=1):
    page_info = ApplicationPage('Europe', "continent")
    page_data = EcdcContinent.get_all_as_page(page)
    return render_template(
        'ecdc/continent/ecdc_continent_all.html',
        page_data=page_data,
        page_info=page_info)


@app_ecdc.route('/continent/<int:continent_id>/page/<int:page>')
@app_ecdc.route('/continent/<int:continent_id>')
def url_ecdc_continent_one(continent_id, page=1):
    page_info = ApplicationPage('Europe', "continent")
    continent = EcdcContinent.get_by_id(continent_id)
    page_data = EcdcCountry.find_by_continent(continent, page)
    return render_template(
        'ecdc/continent/ecdc_continent_one.html',
        continent=continent,
        page_data=page_data,
        page_info=page_info)


@app_ecdc.route('/country/all/page/<int:page>')
@app_ecdc.route('/country/all')
def url_ecdc_country_all(page=1):
    page_info = ApplicationPage('Europe', "country")
    page_data = EcdcCountry.get_all_as_page(page)
    return render_template(
        'ecdc/country/ecdc_country_all.html',
        page_data=page_data,
        page_info=page_info)


@app_ecdc.route('/country/<int:country_id>/page/<int:page>')
@app_ecdc.route('/country/<int:country_id>')
def url_ecdc_country_one(country_id, page=1):
    page_info = ApplicationPage('Europe', "country")
    europe_country = EcdcCountry.get_by_id(country_id)
    page_data = EcdcData.find_by_country(europe_country, page)
    return render_template(
        'ecdc/country/ecdc_country_one.html',
        europe_country=europe_country,
        page_data=page_data,
        page_info=page_info)


@app_ecdc.route('/country/germany/page/<int:page>')
@app_ecdc.route('/country/germany')
def url_ecdc_country_germany(page=1):
    page_info = ApplicationPage('Europe', "country: Germany")
    europe_country = EcdcCountry.get_germany()
    if europe_country is None:
        flash('country: Germany not found in Database', category='error')
        return redirect(url_for('ecdc.url_ecdc_tasks'))
    page_data = EcdcData.find_by_country(europe_country, page)
    return render_template(
        'ecdc/country/ecdc_country_germany.html',
        europe_country=europe_country,
        page_data=page_data,
        page_info=page_info)


# ----------------------------------------------------------------------------------------------------------------
#  Celery TASKS
# ----------------------------------------------------------------------------------------------------------------


@celery.task(bind=True)
def task_ecdc_import(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_ecdc_import [OK] ")
    logger.info("------------------------------------------------------------")
    ecdc_service.import_file()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_ecdc_import)"
    return result


@celery.task(bind=True)
def task_ecdc_full_update_dimension_tables(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_ecdc_full_update_dimension_tables [OK] ")
    logger.info("------------------------------------------------------------")
    ecdc_service.full_update_dimension_tables()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_ecdc_full_update_dimension_tables)"
    return result


@celery.task(bind=True)
def task_ecdc_update_dimension_tables(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_ecdc_update_dimension_tables [OK] ")
    logger.info("------------------------------------------------------------")
    ecdc_service.update_dimension_tables()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_ecdc_update_dimension_tables)"
    return result


@celery.task(bind=True)
def task_ecdc_full_update_fact_table(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_ecdc_full_update_fact_table [OK] ")
    logger.info("------------------------------------------------------------")
    ecdc_service.full_update_fact_table()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_ecdc_full_update_fact_table)"
    return result


@celery.task(bind=True)
def task_ecdc_update_fact_table(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_ecdc_update_fact_table [OK] ")
    logger.info("------------------------------------------------------------")
    ecdc_service.update_fact_table()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_ecdc_update_fact_table)"
    return result


@celery.task(bind=True)
def task_ecdc_full_update_fact_table(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_ecdc_full_update_fact_table [OK] ")
    logger.info("------------------------------------------------------------")
    ecdc_service.full_update_fact_table()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_ecdc_full_update_fact_table)"
    return result


@celery.task(bind=True)
def task_ecdc_full_update_star_schema(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_ecdc_full_update_star_schema [OK] ")
    logger.info("------------------------------------------------------------")
    ecdc_service.full_update_star_schema()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_ecdc_update_star_schema_incremental)"
    return result


@celery.task(bind=True)
def task_ecdc_update_star_schema(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_ecdc_update_star_schema [OK] ")
    logger.info("------------------------------------------------------------")
    ecdc_service.full_update_star_schema()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_ecdc_update_star_schema)"
    return result


@celery.task(bind=True)
def task_ecdc_full_update_star_schema(self):
    logger = get_task_logger(__name__)
    self.update_state(state=states.STARTED)
    logger.info("------------------------------------------------------------")
    logger.info(" Received: task_ecdc_full_update_star_schema [OK] ")
    logger.info("------------------------------------------------------------")
    ecdc_service.full_update_star_schema()
    self.update_state(state=states.SUCCESS)
    result = "OK (task_ecdc_full_update_star_schema)"
    return result

# ----------------------------------------------------------------------------------------------------------------
#  URL Routes for Celery TASKS
# ----------------------------------------------------------------------------------------------------------------


@app_ecdc.route('/task/download')
def url_ecdc_task_download():
    flash("url_ecdc_task_download started")
    ecdc_service.download()
    return redirect(url_for('ecdc.url_ecdc_tasks'))


@app_ecdc.route('/task/import')
def url_ecdc_task_import():
    flash("url_ecdc_task_import started")
    task_ecdc_import.apply_async()
    return redirect(url_for('ecdc.url_ecdc_tasks'))


@app_ecdc.route('/task/update/dimension-tables')
def url_ecdc_task_update_dimensiontables():
    flash("url_ecdc_task_update_dimensiontables started")
    task_ecdc_update_dimension_tables.apply_async()
    return redirect(url_for('ecdc.url_ecdc_tasks'))


@app_ecdc.route('/task/update/full/dimension-tables')
def url_ecdc_task_full_update_dimensiontables():
    flash("url_ecdc_task_full_update_dimensiontables started")
    task_ecdc_full_update_dimension_tables.apply_async()
    return redirect(url_for('ecdc.url_ecdc_tasks'))


@app_ecdc.route('/task/update/fact-table')
def url_task_ecdc_update_fact_table():
    flash("url_ecdc_task_update_facttable started")
    task_ecdc_update_fact_table.apply_async()
    return redirect(url_for('ecdc.url_ecdc_tasks'))


@app_ecdc.route('/task/full/update/fact-table')
def url_task_ecdc_full_update_fact_table():
    flash("url_ecdc_task_full_update_facttable started")
    task_ecdc_full_update_fact_table.apply_async()
    return redirect(url_for('ecdc.url_ecdc_tasks'))


@app_ecdc.route('/task/update/full/star_schema')
def url_task_ecdc_full_update_star_schema():
    flash("url_ecdc_task_full_update_star_schema started")
    ecdc_service.download()
    task_ecdc_full_update_star_schema.apply_async()
    return redirect(url_for('ecdc.url_ecdc_tasks'))


@app_ecdc.route('/task/update/star_schema')
def url_task_ecdc_update_star_schema():
    flash("url_ecdc_task_update_starschema started")
    ecdc_service.download()
    task_ecdc_update_star_schema.apply_async()
    return redirect(url_for('ecdc.url_ecdc_tasks'))
